notebook header, notebook, stack{
    border: none;
    border: 1px solid #111111;
}

notebook, notebook header, stack, list{
    background-color: transparent;
}

#label, dialog label, label:visited, link, #version {
    color: rgb(255, 255, 255);   
}

stack label, tab label{
    color:white;
}

window grid grid {
    padding-top: 15px;
    padding-left: 40px;
    padding-right: 40px;
}

#settings>box>grid, #dialog>box{
    /* border: 1px red solid; */
    padding:20px;
}

#settings separator {
    color: white;
    background-color: white;
}
#settings button, #warn button label, #settings spinbutton{
    background-color: white;
}
#warn button label, #settings button label {
    color: black;
}

#warn>box{
    padding:10px;
}

/* TABS COLOR */

header > tabs > tab{
    background-color: rgb(148, 24, 24);
}

header > tabs > tab:hover, header > tabs > tab:checked{
    background-color: rgb(100, 21, 21);
    
}

/* RADIOS */

radiobutton {
    color: white;
    padding-top: 15px;
    padding-bottom: 25px;
}

radiobutton:checked, radiobutton:active{
    border: none;
}

/* NAMES */

#buttons {
    margin-bottom: 10px;
}

#info {
    margin-right: 60px;
    margin-left: 60px;
}

#top {
    padding-top: 20px;
}

#invisible {
    opacity: 0;
}

/* CLASSES */

.help, .close, #version {

    padding: 10px;

}

#link:hover, #link:active {
    background: transparent;

}

.cambModos {
 
    margin: 20px 0;
}
.cambModos2{
    color: rgb(224,28,52);
    font-family: monospace;
    font-size: xx-large;
    font-weight: bolder;
    margin: 20px 0;
}

.bg-color, dialog {
    background-repeat: no-repeat;
    background-color: #111111;
}

.bg-image {
    background-image: url('../images/bg4.png');
    background-repeat: no-repeat;
    background-size: auto;
    background-color: #111111;
}

.button-none{
    opacity: 0;
}

/* GPU */
.box-gpu-property {
    box-shadow: -0.4em 0 0.4em rgb(100, 21, 21);
    padding-left: 5px;
}